@@ -7,21 +7,23 @@ for pkg in {"servo", "servox"}:
     except importlib.metadata.PackageNotFoundError:
         pass
 
+__codename__ = "pass the calimari"
+
 import servo.assembly
 import servo.connector
+import servo.connectors
 import servo.errors
 import servo.events
 import servo.types
 import servo.cli
 import servo.utilities
-import servo.logging
 
 # Import the core classes
 # These are what most developers will need
 from .events import (
     Event,
     EventHandler,
-    EventResult, 
+    EventResult,
     Preposition,
     create_event,
     event,
@@ -30,15 +32,16 @@ from .events import (
     after_event,
     event_handler,
 )
-from .connector import (    
-    AbstractBaseConfiguration,
+from .connector import (
     BaseConfiguration,
     BaseConnector,
     Optimizer,
     metadata,
 )
 
+from .configuration import *
 from .checks import *
 from .errors import *
+from .logging import *
 from .types import *
 from .utilities import *
