@@ -1,5 +1,4 @@
 from __future__ import annotations
-import asyncio
 import abc
 import importlib
 import re
@@ -8,6 +7,7 @@ from typing import (
     Any,
     ClassVar,
     IO,
+    Iterable,
     Generator,
     Optional,
     Set,
@@ -26,17 +26,10 @@ from pydantic import (
 
 
 from servo import api, events, logging, repeating
-from servo.configuration import AbstractBaseConfiguration, BaseConfiguration, Optimizer
+from servo.configuration import BaseConfiguration, Optimizer
 from servo.events import EventHandler, EventResult
 from servo.types import *
-from servo.utilities import (
-    OutputStreamCallback,
-    SubprocessResult,
-    Timeout,
-    run_subprocess_shell, 
-    stream_subprocess_shell,
-    associations
-)
+from servo.utilities import associations
 
 
 _connector_subclasses: Set[Type["BaseConnector"]] = set()
@@ -47,7 +40,7 @@ class BaseConnector(associations.Mixin, api.Mixin, events.Mixin, logging.Mixin,
     """
     Connectors expose functionality to Servo assemblies by connecting external services and resources.
     """
-    
+
     ##
     # Connector metadata
 
@@ -109,7 +102,7 @@ class BaseConnector(associations.Mixin, api.Mixin, events.Mixin, logging.Mixin,
         assert isinstance(
             cls.version, Version
         ), "version is not a semantic versioning descriptor"
-        
+
         if not cls.__default_name__:
             if name := _name_for_connector_class(cls):
                 cls.__default_name__ = name
@@ -128,8 +121,8 @@ class BaseConnector(associations.Mixin, api.Mixin, events.Mixin, logging.Mixin,
     @classmethod
     def config_model(cls) -> Type["BaseConfiguration"]:
         """
-        Return the configuration model backing the connector. 
-        
+        Return the configuration model backing the connector.
+
         The model is determined by the type hint of the `configuration` attribute
         nearest in definition to the target class in the inheritance hierarchy.
         """
@@ -141,7 +134,7 @@ class BaseConnector(associations.Mixin, api.Mixin, events.Mixin, logging.Mixin,
         super().__init_subclass__(**kwargs)
 
         _connector_subclasses.add(cls)
-        
+
         cls.name = cls.__name__.replace("Connector", "")
         cls.full_name = cls.__name__.replace("Connector", " Connector")
         cls.version = Version.parse("0.0.0")
@@ -158,121 +151,21 @@ class BaseConnector(associations.Mixin, api.Mixin, events.Mixin, logging.Mixin,
         )
         super().__init__(
             *args, name=name, **kwargs,
-        )        
+        )
 
     def __hash__(self):
         return hash((self.name, id(self),))
-    
+
     @property
     def api_client_options(self) -> Dict[str, Any]:
         return self.__dict__.get("api_client_options", super().api_client_options)
-    
-    ##
-    # Subprocess utilities
 
-    async def run_subprocess(
-        self,
-        cmd: str,
-        *,
-        env: Optional[Dict[str, str]] = None,
-        timeout: Timeout = None,
-        logger: Optional[loguru.Logger] = ...,
-        stdin: Union[int, IO[Any], None] = None,
-        **kwargs
-    ) -> SubprocessResult:
-        """
-        Run a shell command in a subprocess asynchronously and return the results.
-
-        This method is a convenience wrapper for lower-level functionality implemented in the
-        `servo.utilities.subprocess` module. Additional arguments are available for input via
-        `kwargs` that have been omitted for brevity to optimize the public API for common cases.
-
-        :param cmd: The command to run.
-        :param env: An optional dictionary of environment variables to apply to the subprocess.
-        :param timeout: An optional timeout in seconds for how long to read the streams before giving up.
-        :param logger: The logger to log messages about the subprocess execution against. Defaults to `self.logger`. `None` disables logging output.
-        :param stdin: A file descriptor, IO stream, or None value to use as the standard input of the subprocess. Default is `None`.
-
-        :raises asyncio.TimeoutError: Raised if the timeout expires before the subprocess exits.
-        :return: A named tuple value of the exit status and two string lists of standard output and standard error.
-        """
-        logger_: Optional[loguru.Logger] = self.logger if logger == ... else logger
-        try:
-            start = time.time()
-            if logger_:
-                timeout_note = f" ({Duration(timeout)} timeout)" if timeout else ""
-                logger_.info(f"Running subprocess command `{cmd}`{timeout_note}")
-            result = await run_subprocess_shell(
-                cmd,           
-                env=env,
-                timeout=timeout,
-                stdin=stdin,
-                **kwargs
-            )
-            end = time.time()
-            duration = Duration(end - start)
-            if logger_:
-                logger_.info(f"Subprocess finished with return code {result.return_code} in {duration} (`{cmd}`)")
-            return result
-        except asyncio.TimeoutError as error:
-            if logger_:
-                logger_.warning(f"timeout expired waiting for subprocess to complete: {error}")
-            raise error
-
-    async def stream_subprocess_output(
-        self,
-        cmd: str,
-        *,
-        stdout_callback: Optional[OutputStreamCallback] = None,
-        stderr_callback: Optional[OutputStreamCallback] = None,
-        env: Optional[Dict[str, str]] = None,
-        timeout: Timeout = None,        
-        stdin: Union[int, IO[Any], None] = None,
-        logger: Optional[loguru.Logger] = ...,
-        **kwargs
-    ) -> int:
-        """
-        Run a shell command in a subprocess asynchronously and stream its output.
-
-        This method is a convenience wrapper for lower-level functionality implemented in the
-        `servo.utilities.subprocess` module. Additional arguments are available for input via
-        `kwargs` that have been omitted for brevity to optimize the public API for common cases.
-
-        :param cmd: The command to run.
-        :param stdout_callback: An optional callable invoked with each line read from stdout. Must accept a single string positional argument and returns nothing.
-        :param stderr_callback: An optional callable invoked with each line read from stderr. Must accept a single string positional argument and returns nothing.
-        :param env: An optional dictionary of environment variables to apply to the subprocess.
-        :param timeout: An optional timeout in seconds for how long to read the streams before giving up.
-        :param logger: The logger to log messages about the subprocess execution against. Defaults to `self.logger`. `None` disables logging output.
-        :param stdin: A file descriptor, IO stream, or None value to use as the standard input of the subprocess. Default is `None`.
-
-        :raises asyncio.TimeoutError: Raised if the timeout expires before the subprocess exits.
-        :return: The exit status of the subprocess.
-        """
-        logger_: Optional[loguru.Logger] = self.logger if logger == ... else logger
-        try:
-            start = time.time()
-            if logger_:
-                timeout_note = f" ({Duration(timeout)} timeout)" if timeout else ""
-                logger_.info(f"Running subprocess command `{cmd}`{timeout_note}")
-            result = await stream_subprocess_shell(
-                cmd,
-                stdout_callback=stdout_callback,
-                stderr_callback=stderr_callback,                
-                env=env,
-                timeout=timeout,
-                stdin=stdin,
-                **kwargs
-            )
-            end = time.time()
-            duration = Duration(end - start)
-            if logger_:
-                logger_.info(f"Subprocess finished with return code {result} in {duration} (`{cmd}`)")
-            return result
-        except asyncio.TimeoutError as error:
-            if logger_:
-                logger_.warning(f"timeout expired waiting for subprocess to complete: {error}")
-            raise error
+    @property
+    def logger(self) -> loguru.Logger:
+        """Returns a contextualized logger"""
+        # NOTE: We support the explicit connector ref and the context var so
+        # that logging is attributable outside of an event whenever possible
+        return super().logger.bind(connector=self)
 
 
 EventResult.update_forward_refs(BaseConnector=BaseConnector)
@@ -298,7 +191,7 @@ def metadata(
             if isinstance(name, tuple):
                 if len(name) != 2:
                     raise ValueError(f"Connector names given as tuples must contain exactly 2 elements: full name and alias")
-                cls.name, cls.__default_name__ = name                
+                cls.name, cls.__default_name__ = name
             else:
                 cls.name = name
         if description:
@@ -415,7 +308,7 @@ def _routes_for_connectors_descriptor(connectors) -> Dict[str, "BaseConnector"]:
         connector_routes = {}
         for name, value in connectors.items():
             if not isinstance(name, str):
-                raise TypeError(f'Connector names must be strings: "{key}"')
+                raise TypeError(f'Connector names must be strings: "{name}"')
 
             # Validate the name
             try:
@@ -428,6 +321,8 @@ def _routes_for_connectors_descriptor(connectors) -> Dict[str, "BaseConnector"]:
                 connector_class = value
             elif isinstance(value, str):
                 connector_class = _connector_class_from_string(value)
+            else:
+                raise ValueError(f'"{value}" is not a string or type')
 
             # Check for key reservations
             if name in _reserved_keys():
@@ -461,7 +356,7 @@ def _connector_class_from_string(connector: str) -> Optional[Type["BaseConnector
         )
     except Exception:
         pass
-    
+
     if _validate_class(connector_class):
         return connector_class
 
