@@ -18,15 +18,17 @@ from servo import (
     Description,
     Duration,
     Filter,
-    HaltOnFailed,
     License,
     Maturity,
     Measurement,
     Metric,
+    Severity,
     Unit,
     check,
+    logger,
     metadata,
     on_event,
+    require,
     TimeSeries,
     DurationProgress,
 )
@@ -39,7 +41,7 @@ API_PATH = "/api/v1"
 
 
 class PrometheusMetric(Metric):
-    """PrometheusMetric objects describe metrics that can be measure by querying
+    """PrometheusMetric objects describe metrics that can be measured by querying
 Prometheus.
     """
     query: str
@@ -49,10 +51,10 @@ Prometheus.
     Querying](https://prometheus.io/docs/prometheus/latest/querying/basics/)
     documentation.
     """
-    
+
     step: Duration = "1m"
     """The resolution of the query.
-    
+
     The step resolution determines the number of data points captured across a
     query range.
     """
@@ -70,7 +72,7 @@ class PrometheusTarget(BaseModel):
 
 class PrometheusConfiguration(BaseConfiguration):
     """PrometheusConfiguration objects describe how PrometheusConnector objects
-capture measurements from the Prometheus metrics server. 
+capture measurements from the Prometheus metrics server.
     """
 
     base_url: AnyHttpUrl = DEFAULT_BASE_URL
@@ -114,7 +116,7 @@ capture measurements from the Prometheus metrics server.
             ],
             **kwargs,
         )
-    
+
     @validator("base_url", allow_reuse=True)
     @classmethod
     def rstrip_base_url(cls, base_url):
@@ -130,18 +132,18 @@ class PrometheusRequest(BaseModel):
     metric: PrometheusMetric
     start: datetime
     end: datetime
-    
+
     @property
     def query(self) -> str:
         return self.metric.query
-    
+
     @property
     def step(self) -> Duration:
-        return self.metric.step 
+        return self.metric.step
 
     @property
     def url(self) -> str:
-        return "".join(self.base_url.rstrip("/") + 
+        return "".join(self.base_url.rstrip("/") +
             "/query_range" +
             f"?query={self.query}" +
             f"&start={self.start.timestamp()}" +
@@ -155,14 +157,14 @@ determine if it is ready for use in an optimization run.
     """
     config: PrometheusConfiguration
 
-    @check("Connect to \"{self.config.base_url}\"", required=True)
+    @require("Connect to \"{self.config.base_url}\"")
     async def check_base_url(self) -> None:
         """Checks that the Prometheus base URL is valid and reachable.
         """
         async with httpx.AsyncClient(base_url=self.config.api_url) as client:
             response = await client.get("targets")
             response.raise_for_status()
-    
+
     @multicheck("Run query \"{item.query}\"")
     async def check_queries(self) -> Tuple[Iterable, CheckHandler]:
         """Checks that all metrics have valid, well-formed PromQL queries.
@@ -170,16 +172,16 @@ determine if it is ready for use in an optimization run.
         async def query_for_metric(metric: PrometheusMetric) -> str:
             start, end = datetime.now() - timedelta(minutes=10), datetime.now()
             prometheus_request = PrometheusRequest(base_url=self.config.api_url, metric=metric, start=start, end=end)
-        
-            self.logger.trace(f"Querying Prometheus (`{metric.query}`): {prometheus_request.url}")
+
+            logger.trace(f"Querying Prometheus (`{metric.query}`): {prometheus_request.url}")
             async with httpx.AsyncClient() as client:
                 response = await client.get(prometheus_request.url)
                 response.raise_for_status()
                 result = response.json()
                 return f"returned {len(result)} results"
-        
+
         return self.config.metrics, query_for_metric
-    
+
     @check("Active targets")
     async def check_targets(self) -> str:
         """Checks that all targets are being scraped by Prometheus and report as
@@ -189,14 +191,11 @@ determine if it is ready for use in an optimization run.
             response = await client.get("/api/v1/targets")
             response.raise_for_status()
             result = response.json()
-        
-        debug(result)
+
         target_count = len(result["data"]["activeTargets"])
         assert target_count > 0
-
-
         return f"found {target_count} targets"
-    
+
 @metadata(
     description="Prometheus Connector for Opsani",
     version="1.5.0",
@@ -211,11 +210,11 @@ class PrometheusConnector(BaseConnector):
     config: PrometheusConfiguration
 
     @on_event()
-    async def check(self, 
-        filter_: Optional[Filter] = None, 
-        halt_on: HaltOnFailed = HaltOnFailed.requirement
+    async def check(self,
+        filter_: Optional[Filter] = None,
+        halt_on: Optional[Severity] = Severity.critical
     ) -> List[Check]:
-        """Checks that the configuration is valid and the connector can capture        
+        """Checks that the configuration is valid and the connector can capture
         measurements from Prometheus.
 
         Checks are implemented in the PrometheusChecks class.
@@ -223,13 +222,13 @@ class PrometheusConnector(BaseConnector):
         Args:
             filter_ (Optional[Filter], optional): A filter for limiting the
                 checks that are run. Defaults to None.
-            halt_on (HaltOnFailed, optional): When to halt running checks.
-                Defaults to HaltOnFailed.requirement.
+            halt_on (Severity, optional): When to halt running checks.
+                Defaults to Severity.critical.
 
         Returns:
-            List[Check]: A list of check objects that report the outcomes of the            
+            List[Check]: A list of check objects that report the outcomes of the
                 checks that were run.
-        """        
+        """
         return await PrometheusChecks.run(self.config, filter_, halt_on=halt_on)
 
     @on_event()
@@ -261,9 +260,9 @@ class PrometheusConnector(BaseConnector):
         optimizer.
 
         Args:
-            metrics (List[str], optional): A list of the metric names to measure. 
+            metrics (List[str], optional): A list of the metric names to measure.
                 When None, all configured metrics are measured. Defaults to None.
-            control (Control, optional): A control descriptor that describes how            
+            control (Control, optional): A control descriptor that describes how
                 the measurement is to be captured. Defaults to Control().
 
         Returns:
@@ -303,13 +302,13 @@ class PrometheusConnector(BaseConnector):
         self, metric: PrometheusMetric, start: datetime, end: datetime
     ) -> List[TimeSeries]:
         prometheus_request = PrometheusRequest(base_url=self.config.api_url, metric=metric, start=start, end=end)
-        
+
         self.logger.trace(f"Querying Prometheus (`{metric.query}`): {prometheus_request.url}")
         async with self.api_client() as client:
             try:
                 response = await client.get(prometheus_request.url)
                 response.raise_for_status()
-            except (httpx.HTTPError, httpcore._exceptions.ReadTimeout, httpcore._exceptions.ConnectError) as error:                
+            except (httpx.HTTPError, httpcore._exceptions.ReadTimeout, httpcore._exceptions.ConnectError) as error:
                 self.logger.trace(f"HTTP error encountered during GET {prometheus_request.url}: {error}")
                 raise
 
@@ -318,7 +317,7 @@ class PrometheusConnector(BaseConnector):
 
         if "status" not in data or data["status"] != "success":
             return []
-    
+
         readings = []
         for result_dict in data["data"]["result"]:
             m_ = result_dict["metric"].copy()
