@@ -22,24 +22,25 @@ from servo import (
     Duration,
     DurationProgress,
     Filter,
-    HaltOnFailed,
     License,
     Maturity,
     Measurement,
     Metric,
     Numeric,
+    Severity,
     TimeSeries,
     Unit,
     check,
     cli,
+    logger,
     metadata,
     on_event,
+    require,
+    stream_subprocess_shell,
     values_for_keys,
     value_for_key_path,
 )
 
-###
-### Vegeta
 
 METRICS = [
     Metric("throughput", Unit.REQUESTS_PER_MINUTE),
@@ -125,7 +126,7 @@ class VegetaConfiguration(BaseConfiguration):
         description="Specifies the request rate per time unit to issue against the targets. Given in the format of request/time unit.",
     )
     duration: Duration = Field(
-        description="Specifies the amount of time to issue requests to the targets.",
+        description="Specifies the amount of time to issue requests to the targets. This value can be overridden by the server.",
     )
     format: TargetFormat = Field(
         TargetFormat.http,
@@ -145,8 +146,8 @@ class VegetaConfiguration(BaseConfiguration):
         10,
         description="Specifies the initial number of workers used in the attack. The workers will automatically increase to achieve the target request rate, up to max-workers.",
     )
-    max_workers: int = Field(
-        18446744073709551615,
+    max_workers: Optional[int] = Field(
+        None,
         description="The maximum number of workers used to sustain the attack. This can be used to control the concurrency of the attack to simulate a target number of clients.",
     )
     max_body: int = Field(
@@ -290,23 +291,20 @@ class VegetaConfiguration(BaseConfiguration):
             {TargetFormat: lambda t: t.value()}
         )
 
-VegetaRunner = Callable[[VegetaConfiguration], Awaitable[Tuple[int, List[VegetaReport]]]]
-
 class VegetaChecks(BaseChecks):
     config: VegetaConfiguration
-    runner: VegetaRunner
     reports: Optional[List[VegetaReport]] = None
 
-    @check("Vegeta execution", required=True)
+    @require("Vegeta execution")
     async def check_execution(self) -> Tuple[bool, str]:
-        exit_code, reports = await self.runner(config=self.config)
+        exit_code, reports = await _run_vegeta(config=self.config)
         self.reports = reports
         return (exit_code == 0, f"Vegeta exit code: {exit_code}")
-    
+
     @check("Report aggregation")
     def check_report_aggregation(self) -> Tuple[bool, str]:
         return (len(self.reports) > 0, f"Collected {len(self.reports)} reports")
-    
+
     @check("Error rate < 5.0%")
     def check_error_rates(self) -> Tuple[bool, str]:
         vegeta_report = self.reports[-1]
@@ -320,13 +318,12 @@ class VegetaChecks(BaseChecks):
     maturity=Maturity.STABLE,
 )
 class VegetaConnector(BaseConnector):
-    config: VegetaConfiguration    
-    warmup_until: Optional[datetime] = None
+    config: VegetaConfiguration
 
     @on_event()
     def describe(self) -> Description:
         """
-        Describe the metrics and components exported by the connector.
+        Describes the metrics and components exported by the connector.
         """
         return Description(metrics=METRICS, components=[])
 
@@ -335,21 +332,19 @@ class VegetaConnector(BaseConnector):
         return METRICS
 
     @on_event()
-    async def check(self, filter_: Optional[Filter] = None, halt_on: HaltOnFailed = HaltOnFailed.requirement) -> List[Check]:
+    async def check(self, filter_: Optional[Filter] = None, halt_on: Optional[Severity] = Severity.critical) -> List[Check]:
         # Take the current config and run a 5 second check against it
-        self.warmup_until = datetime.now()
         check_config = self.config.copy()
         check_config.duration = "5s"
         check_config.reporting_interval = "1s"
 
-        checks = VegetaChecks(config=check_config, runner=self._run_vegeta)
-        return await checks.run_(filter_, halt_on=halt_on)
+        return await VegetaChecks.run(check_config, filter_, halt_on=halt_on)
 
     @on_event()
     async def measure(
         self, *, metrics: List[str] = None, control: Control = Control()
     ) -> Measurement:
-        self.warmup_until = datetime.now() + control.warmup
+        warmup_until = datetime.now() + control.warmup
 
         number_of_urls = (
             1 if self.config.target else _number_of_lines_in_file(self.config.targets)
@@ -358,7 +353,7 @@ class VegetaConnector(BaseConnector):
         self.logger.info(summary)
 
         # Run the load generation
-        _, vegeta_reports = await self._run_vegeta()
+        _, vegeta_reports = await _run_vegeta(config=self.config, warmup_until=warmup_until)
 
         self.logger.info(
             f"Producing time series readings from {len(vegeta_reports)} Vegeta reports"
@@ -375,45 +370,45 @@ class VegetaConnector(BaseConnector):
 
         return measurement
 
-    async def _run_vegeta(
-        self, *, config: Optional[VegetaConfiguration] = None
-    ) -> Tuple[int, List[VegetaReport]]:
-        vegeta_reports: List[VegetaReport] = []
-        config = config if config else self.config
-        vegeta_cmd = _build_vegeta_command(config)
-        ansi_escape = re.compile(r"(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]")
-        progress = DurationProgress(config.duration)
-        
-        async def process_stdout(output: str) -> None:
-            json_report = ansi_escape.sub("", output)
-            vegeta_report = VegetaReport(**json.loads(json_report))
+async def _run_vegeta(
+    config: VegetaConfiguration,
+    warmup_until: Optional[datetime] = None
+) -> Tuple[int, List[VegetaReport]]:
+    vegeta_reports: List[VegetaReport] = []
+    vegeta_cmd = _build_vegeta_command(config)
+    ansi_escape = re.compile(r"(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]")
+    progress = DurationProgress(config.duration)
 
-            if datetime.now() > self.warmup_until:
-                if not progress.started:
-                    progress.start()
+    async def process_stdout(output: str) -> None:
+        json_report = ansi_escape.sub("", output)
+        vegeta_report = VegetaReport(**json.loads(json_report))
 
-                vegeta_reports.append(vegeta_report)
-                summary = _summarize_report(vegeta_report, config)
-                self.logger.info(progress.annotate(summary), progress=progress.progress)
-            else:
-                self.logger.debug(
-                    f"Vegeta metrics excluded (warmup in effect): {vegeta_report}"
-                )
-        
-        self.logger.debug(f"Vegeta started: `{vegeta_cmd}`")
-        exit_code = await self.stream_subprocess_output(
-            vegeta_cmd,
-            stdout_callback=process_stdout,
-            stderr_callback=lambda m: self.logger.error(f"Vegeta stderr: {m}")
+        if warmup_until is None or datetime.now() > warmup_until:
+            if not progress.started:
+                progress.start()
+
+            vegeta_reports.append(vegeta_report)
+            summary = _summarize_report(vegeta_report, config)
+            logger.info(progress.annotate(summary), progress=progress.progress)
+        else:
+            logger.debug(
+                f"Vegeta metrics excluded (warmup in effect): {vegeta_report}"
+            )
+
+    logger.debug(f"Vegeta started: `{vegeta_cmd}`")
+    exit_code = await stream_subprocess_shell(
+        vegeta_cmd,
+        stdout_callback=process_stdout,
+        stderr_callback=lambda m: logger.error(f"Vegeta stderr: {m}")
+    )
+
+    logger.debug(f"Vegeta exited with exit code: {exit_code}")
+    if exit_code != 0:
+        logger.error(
+            f"Vegeta command `{vegeta_cmd}` failed with exit code {exit_code}"
         )
 
-        self.logger.debug(f"Vegeta exited with exit code: {exit_code}")
-        if exit_code != 0:
-            self.logger.error(
-                f"Vegeta command `{vegeta_cmd}` failed with exit code {exit_code}"
-            )        
-
-        return exit_code, vegeta_reports
+    return exit_code, vegeta_reports
 
 
 def _build_vegeta_command(config: VegetaConfiguration) -> str:
@@ -436,7 +431,7 @@ def _build_vegeta_command(config: VegetaConfiguration) -> str:
                 "-workers",
                 config.workers,
                 "-max-workers",
-                config.max_workers,
+                config.max_workers or 18446744073709551615,
                 "-http2",
                 config.http2,
                 "-keepalive",
