@@ -1,3 +1,9 @@
+"""The `servo.logging` module provides standard logging capabilities to the
+servo package and its dependencies.
+
+Logging is implemented on top of the
+[loguru](https://loguru.readthedocs.io/en/stable/) library.
+"""
 from __future__ import annotations
 import asyncio
 import functools
@@ -5,21 +11,16 @@ import logging
 import sys
 import time
 import traceback
-from datetime import datetime
 from pathlib import Path
 from typing import Awaitable, Any, Callable, Dict, Optional, Set, Union, cast
-from weakref import WeakSet
 
 import loguru
-from loguru import logger
-from servo.events import EventContext, _event_context_var
-from servo.types import Duration
+from servo.events import EventContext, _connector_context_var, _event_context_var
 
 __all__ = (
     "Mixin",
     "Filter",
     "ProgressHandler",
-    "Logger",
     "logger",
     "log_execution",
     "log_execution_time",
@@ -27,12 +28,18 @@ __all__ = (
     "set_level"
 )
 
+# Alias the loguru default logger
+logger = loguru.logger
 
 class Mixin:
+    """The `servo.logging.Mixin` class is a convenience class for adding
+    logging capabilities to arbitrary classes through multiple inheritance.
+    """
     @property
     def logger(self) -> loguru.Logger:
-        """Returns a contextualized logger"""
-        return loguru.logger.bind(connector=self)
+        """Returns the servo package logger"""
+        global logger
+        return logger
 
 
 class Filter:
@@ -55,15 +62,18 @@ class ProgressHandler:
     automatically picked up by the handler and reported back to the API via a callback.
 
     NOTE: We call the logger re-entrantly for misconfigured progress logging attempts. The
-        `progress` should must be excluded on logger calls to avoid recursion.
+        `progress` must be excluded on logger calls to avoid recursion.
     """
     def __init__(self, 
         progress_reporter: Callable[[Dict[Any, Any]], Union[None, Awaitable[None]]], 
-        error_reporter: Callable[[str], Union[None, Awaitable[None]]],
+        error_reporter: Optional[Callable[[str], Union[None, Awaitable[None]]]] = None,
+        exception_handler: Optional[Callable[[Exception], Union[None, Awaitable[None]]]] = None
+
     ) -> None:
         self._progress_reporter = progress_reporter
         self._error_reporter = error_reporter
-        self._tasks = WeakSet()
+        self._exception_handler = exception_handler
+        self._tasks = set()
     
     @property
     def tasks(self) -> Set[asyncio.Task]:
@@ -80,7 +90,8 @@ class ProgressHandler:
         if not progress:
             return
 
-        connector = extra.get("connector", None)
+        # Favor explicit connector in extra (see Mixin) else use the context var
+        connector = extra.get("connector", _connector_context_var.get())
         if not connector:
             return await self._report_error("declining request to report progress for record without a connector attribute", record)
 
@@ -99,7 +110,8 @@ class ProgressHandler:
                 return await self._report_error("declining request to report progress for record without a started_at parameter or inferrable value from event context", record)
 
         connector_name = connector.name if hasattr(connector, "name") else connector
-        return await self._report_progress(
+
+        await self._report_progress(
             operation=operation,
             progress=progress,
             connector=connector_name,
@@ -112,9 +124,25 @@ class ProgressHandler:
         """
         Report progress about a log message that was processed.
         """
+
+        def _handle_task_result(task: asyncio.Task) -> None:
+            try:
+                self._tasks.remove(task)
+                task.result()
+            except asyncio.CancelledError:
+                pass  # Task cancellation should not be logged as an error.
+            except Exception as error:  # pylint: disable=broad-except
+                if self._exception_handler:
+                    if asyncio.iscoroutinefunction(self._exception_handler):
+                        asyncio.create_task(self._exception_handler(error))
+                    else:
+                        self._exception_handler(error)
+
         if self._progress_reporter:
             if asyncio.iscoroutinefunction(self._progress_reporter):
-                self._tasks.add(asyncio.create_task(self._progress_reporter(**kwargs)))
+                task = asyncio.create_task(self._progress_reporter(**kwargs))
+                task.add_done_callback(_handle_task_result)
+                self._tasks.add(task)
             else:
                 self._progress_reporter(**kwargs)
 
@@ -170,7 +198,8 @@ class Formatter:
 
         # Respect an explicit component 
         if not "component" in record["extra"]:        
-            if connector := extra.get("connector", None):
+            # Favor explicit connector from the extra dict or use the context var
+            if connector := extra.get("connector", _connector_context_var.get()):
                 component = connector.name
             else:
                 component = "servo"
@@ -252,7 +281,7 @@ def reset_to_defaults() -> loguru.Logger:
 
 def friendly_decorator(f):
     """
-    A "decorator decorator" that wraps a decorator function such that it can be invoked
+    Returns a "decorator decorator" that wraps a decorator function such that it can be invoked
     with or without parentheses such as:
    
         @decorator(with, arguments, and=kwargs)
@@ -295,6 +324,8 @@ def log_execution_time(func, *, level="DEBUG"):
     """
     @functools.wraps(func)
     def wrapped(*args, **kwargs):
+        from servo.types import Duration
+
         name = func.__name__
         logger_ = logger.opt(depth=1)
 
